@@ -1,5 +1,5 @@
 ;;; Copyright (c) 2005 - 2007, Michael Goffioul (michael dot goffioul at swing dot be)
-;;; Copyright (c) 2008 - 2011, Juan Jose Garcia Ripoll
+;;; Copyright (c) 2008 - 2012, Juan Jose Garcia Ripoll
 ;;;
 ;;;   This program is free software; you can redistribute it and/or
 ;;;   modify it under the terms of the GNU Library General Public
@@ -36,7 +36,9 @@
   ((type :reader bundle-op-type)
    (monolithic :initform nil :reader bundle-op-monolithic-p)
    (name-suffix :initarg :name-suffix :initform nil)
-   (build-args :initarg :args :initform nil :accessor bundle-op-build-args)))
+   (build-args :initarg :args :initform nil :accessor bundle-op-build-args)
+   (lisp-files :initform nil :accessor bundle-op-lisp-files)))
+
 
 (defclass fasl-op (bundle-op)
   ((type :initform :fasl)))
@@ -72,10 +74,12 @@
           (if (bundle-op-monolithic-p instance) "-mono" "")))
   (when (typep instance 'monolithic-bundle-op)
     (destructuring-bind (&rest original-initargs
-                         &key prologue-code epilogue-code &allow-other-keys)
+			 &key lisp-files prologue-code epilogue-code
+			 &allow-other-keys)
         (slot-value instance 'original-initargs)
       (setf (slot-value instance 'original-initargs)
-            (remove-keys '(epilogue-code prologue-code) original-initargs)
+            (remove-keys '(lisp-files epilogue-code prologue-code) original-initargs)
+            (bundle-op-lisp-files instance) lisp-files
             (monolithic-op-prologue-code instance) prologue-code
             (monolithic-op-epilogue-code instance) epilogue-code)))
   (setf (bundle-op-build-args instance)
@@ -167,9 +171,9 @@
   (list (cons (make-instance 'lib-op) c)))
 
 (defmethod component-depends-on ((o bundle-op) (c system))
-  (loop for (op . dep) in (bundle-sub-operations o c)
-     when (typep dep 'system)
-     collect (list (class-name (class-of op))
+  (loop :for (op . dep) :in (bundle-sub-operations o c)
+    :when (typep dep 'system)
+    :collect (list (class-name (class-of op))
                    (component-name dep))))
 
 (defmethod component-depends-on ((o lib-op) (c system))
@@ -181,8 +185,8 @@
   nil)
 
 (defmethod input-files ((o bundle-op) (c system))
-  (loop for (sub-op . sub-c) in (bundle-sub-operations o c)
-     nconc (output-files sub-op sub-c)))
+  (loop :for (sub-op . sub-c) :in (bundle-sub-operations o c)
+    :nconc (output-files sub-op sub-c)))
 
 (defmethod output-files ((o bundle-op) (c system))
   (let ((name (concatenate 'base-string (component-name c)
@@ -192,8 +196,8 @@
 
 (defmethod output-files ((o fasl-op) (c system))
   (declare (ignorable o c))
-  (loop for file in (call-next-method)
-     collect (make-pathname :type "fasb" :defaults file)))
+  (loop :for file :in (call-next-method)
+    :collect (make-pathname :type "fasb" :defaults file)))
 
 (defmethod perform ((o bundle-op) (c t))
   (declare (ignorable o c))
@@ -208,7 +212,8 @@
                                :key #'pathname-type :test #'string=))
          (output (output-files o c)))
     (ensure-directories-exist (first output))
-    (apply #'c::builder (bundle-op-type o) (first output) :lisp-files object-files
+    (apply #'c::builder (bundle-op-type o) (first output)
+	   :lisp-files (append object-files (bundle-op-lisp-files o))
            (append (bundle-op-build-args o)
                    (when (and (typep o 'monolithic-bundle-op)
                               (monolithic-op-prologue-code o))
@@ -245,16 +250,16 @@
          (files (and system (output-files operation system))))
     (if (or move-here (and (null move-here-p)
                            (member operation-name '(:program :binary))))
-        (loop with dest-path = (truename (ensure-directories-exist move-here-path))
-           for f in files
-           for new-f = (make-pathname :name (pathname-name f)
+        (loop :with dest-path = (truename (ensure-directories-exist move-here-path))
+          :for f in files
+          :for new-f = (make-pathname :name (pathname-name f)
                                       :type (pathname-type f)
                                       :defaults dest-path)
-           do (progn
+          :do (progn
                 (when (probe-file new-f)
                   (delete-file new-f))
                 (rename-file f new-f))
-           collect new-f)
+           :collect new-f)
         files)))
 
 ;;;
@@ -288,8 +293,8 @@
   (let ((l (input-files o c)))
     (and l
          (load (first l))
-         (loop for i in (module-components c)
-            do (setf (gethash 'load-op (component-operation-times i))
+         (loop :for i :in (module-components c)
+           :do (setf (gethash 'load-op (component-operation-times i))
                      (get-universal-time))))))
 
 ;;;
@@ -299,13 +304,8 @@
 ;;; form. Only useful when the dependencies have also been precompiled.
 ;;;
 
-(defclass compiled-file (component) ())
-(defmethod component-relative-pathname ((component compiled-file))
-  (compile-file-pathname
-   (coerce-pathname
-    (or (slot-value component 'relative-pathname)
-        (component-name component))
-    :type "fas")))
+(defclass compiled-file (component)
+  ((type :initform nil)))
 
 (defmethod output-files (o (c compiled-file))
   (declare (ignore o c))
@@ -331,10 +331,8 @@
 
 (defmethod output-files ((o lib-op) (c prebuilt-system))
   (declare (ignore o))
-  (values (list (compile-file-pathname (prebuilt-system-static-library c)
-                                       :type :lib))
-          t ; Advertise that we do not want this path renamed
-          ))
+  (values (list (prebuilt-system-static-library c))
+          t)) ; Advertise that we do not want this path renamed
 
 (defmethod perform ((o lib-op) (c prebuilt-system))
   (car (output-files o c)))
@@ -376,19 +374,19 @@
                 s))))
 
 (defmethod component-depends-on ((o binary-op) (s system))
-  (loop for dep in (binary-op-dependencies o s)
-     append (apply #'component-depends-on dep)))
+  (loop :for dep :in (binary-op-dependencies o s)
+    :append (apply #'component-depends-on dep)))
 
 (defmethod input-files ((o binary-op) (s system))
-  (loop for dep in (binary-op-dependencies o s)
-     append (apply #'input-files dep)))
+  (loop :for dep :in (binary-op-dependencies o s)
+    :append (apply #'input-files dep)))
 
 (defmethod output-files ((o binary-op) (s system))
   (list* (merge-pathnames* (make-pathname :name (component-name s)
                                           :type "asd")
                            (component-relative-pathname s))
-         (loop for dep in (binary-op-dependencies o s)
-            append (apply #'output-files dep))))
+         (loop :for dep :in (binary-op-dependencies o s)
+           :append (apply #'output-files dep))))
 
 (defmethod perform ((o binary-op) (s system))
   (let* ((dependencies (binary-op-dependencies o s))
@@ -397,8 +395,8 @@
          (filename (first (output-files o s)))
          (name (component-name s))
          (name-keyword (intern (string name) (find-package :keyword))))
-    (loop for dep in dependencies
-       do (apply #'perform dep))
+    (loop :for dep :in dependencies
+      :do (apply #'perform dep))
     (with-open-file (s filename :direction :output :if-exists :supersede
                        :if-does-not-exist :create)
       (format s ";;; Prebuilt ASDF definition for system ~A" name)
@@ -420,17 +418,6 @@
 ;;; Final integration steps
 ;;;
 
-(export '(make-build load-fasl-op prebuilt-system))
-(push '("fasb" . si::load-binary) ext:*load-hooks*)
+(export '(make-build load-fasl-op))
 
-(defun register-pre-built-system (name)
-  (register-system (make-instance 'system :name name :source-file nil)))
-
-(setf ext:*module-provider-functions*
-      (loop :for f :in ext:*module-provider-functions*
-        :unless (eq f 'module-provide-asdf)
-        :collect #'(lambda (name)
-                     (let ((l (multiple-value-list (funcall f name))))
-                       (and (first l) (register-pre-built-system name))
-                       (values-list l)))))
-#+win32 (push '("asd" . si::load-source) ext:*load-hooks*)
+(pushnew '("fasb" . si::load-binary) ext:*load-hooks* :test 'equal :key 'car)
